#include "GL_StaticGroup.h"
#include "X3D_StaticGroup.h"

#include <algorithm>
#include <iostream>

using namespace std;

namespace X3DTK {
namespace GL {

StaticGroup::StaticGroup()
: X3DChildNode()
{  
  defineTypeName("StaticGroup", "Grouping");
}

StaticGroup::StaticGroup(const StaticGroup &G)
: X3DChildNode(G)
{
  MFAbstractNode childrenList = G.getChildList();
  for (MFAbstractNode::const_iterator it = childrenList.begin(); it != childrenList.end(); ++it)
    setChild(*it);
}

SFAbstractNode StaticGroup::clone() const
{
  return new StaticGroup(*this);
}

StaticGroup::~StaticGroup()
{ 
  for (MFNode::iterator it = _children.begin(); it != _children.end(); ++it)
    removeParentFromChild(this, *it); 
}

MFAbstractNode StaticGroup::getChildList() const
{
  MFAbstractNode childrenList;
  for (MFNode::const_iterator it = _children.begin(); it != _children.end(); ++it)
    if (*it != 0)
      childrenList.push_back(*it);
    
  return childrenList; 
}

bool StaticGroup::addChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DChildNode *>(N) != 0)
  {
    MFNode::iterator res = find(_children.begin(), _children.end(), static_cast<SFNode>(N));
    if (res == _children.end())
    {
      _children.push_back(static_cast<SFNode>(N));
      addParentToChild(this, N);
      return true;
    }
    return false;  
  }
  if (N != 0)
    cx3d << "GL::StaticGroup::addChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  else
    cx3d << "GL::StaticGroup::addChild : cannot add a NULL node!" << endl;
   
  
  return false;    
}

bool StaticGroup::setChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DChildNode *>(N) != 0)
  {
    MFNode::iterator res = find(_children.begin(), _children.end(), static_cast<SFNode>(N));
    if (res == _children.end())
    {
      _children.push_back(static_cast<SFNode>(N));
      addParentToChild(this, N);
      return true;
    }
    return false;  
  }
  if (N != 0)
    cx3d << "GL::StaticGroup::setChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  else
    cx3d << "GL::StaticGroup::setChild : cannot add a NULL node!" << endl;
   
  
  return false;   
}

bool StaticGroup::removeChild(const SFAbstractNode &N)
{
  MFNode::iterator res = find(_children.begin(), _children.end(), static_cast<SFNode>(N));
  if (res != _children.end())
  {
    removeParentFromChild(this, *res);
    _children.erase(res);
    return true;
  }  
  return false;
}

}
}
