#include "X3D_StaticGroup.h"

#include <algorithm>
#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

StaticGroup::StaticGroup()
: X3DChildNode(), X3DBoundedObject()
{
  defineTypeName("StaticGroup", "Grouping");
}

StaticGroup::StaticGroup(const SFVec3f &bboxCenter, const SFVec3f &bboxSize)
: X3DChildNode(), X3DBoundedObject(bboxCenter, bboxSize)
{
  defineTypeName("StaticGroup", "Grouping");
}

StaticGroup::StaticGroup(const StaticGroup &G)
: X3DChildNode(G), X3DBoundedObject(G)
{
  MFAbstractNode childrenList = G.getChildList();
  for (MFAbstractNode::const_iterator it = childrenList.begin(); it != childrenList.end(); ++it)
    setChild(*it);
}

SFAbstractNode StaticGroup::clone() const
{
  return new StaticGroup(*this);
}

StaticGroup::~StaticGroup()
{
  for (MFNode::iterator it = _children.begin(); it != _children.end(); ++it)
    removeParentFromChild(this, *it);
}

MFAbstractNode StaticGroup::getChildList() const
{
  MFAbstractNode childrenList;
  for (MFNode::const_iterator it = _children.begin(); it != _children.end(); ++it)
    if (*it != 0)
      childrenList.push_back(*it);
    
  return childrenList; 
}

bool StaticGroup::addChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DChildNode *>(N) != 0)
  {
    _children.push_back(static_cast<SFNode>(N));
    addParentToChild(this, N);
    return true;
  }
  if (N != 0)
    cx3d << "X3D::StaticGroup::addChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  else
    cx3d << "X3D::StaticGroup::addChild : cannot add a NULL node!" << endl;
      
  return false;    
}

bool StaticGroup::setChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DChildNode *>(N) != 0)
  {
    _children.push_back(static_cast<SFNode>(N));
    addParentToChild(this, N);
    return true;
  }
  if (N != 0)
    cx3d << "X3D::StaticGroup::setChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  else
    cx3d << "X3D::StaticGroup::setChild : cannot add a NULL node!" << endl;
  
  return false;    
}

bool StaticGroup::removeChild(const SFAbstractNode &N)
{
  MFNode::iterator res = find(_children.begin(), _children.end(), N);
  if (res != _children.end())
  {
    removeParentFromChild(this, N);
    _children.erase(res);
    return true;
  }
      
  return false;
}

void StaticGroup::load(const X3DFileElement *element)
{
  X3DBoundedObject::load(element);
}

SFString &StaticGroup::write(SFString &output) const
{
  return X3DBoundedObject::write(output);
}

}
}
