#include "GL_RendererShapeVisitor.h"
#include "GL_Material.h"
#include "GL_X3DShapeNode.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace GL {

RendererShapeVisitor::RendererShapeVisitor()
: ShapeVisitor()
{
  // Enter functions.
  define(Recorder<Material>::getEnterFunction(&RendererShapeVisitor::enterMaterial));
  define(Recorder<X3DShapeNode>::getEnterFunction(&RendererShapeVisitor::enterX3DShapeNode));
}

void RendererShapeVisitor::enterMaterial(Material *M) 
{
  RendererStateVariables *stateVariables = Singleton<RendererStateVariables>::getInstance();
  
  glMaterialfv(GL_FRONT, GL_DIFFUSE, &M->getDiffuseColor().front());
  glMaterialfv(GL_FRONT, GL_AMBIENT, &M->getAmbientColor().front());
  glMaterialfv(GL_FRONT, GL_EMISSION, &M->getEmissiveColor().front());
  SFFloat s = M->getShininess();
  glMaterialfv(GL_FRONT, GL_SHININESS, &s); 
  glMaterialfv(GL_FRONT, GL_SPECULAR, &M->getSpecularColor().front());
}

void RendererShapeVisitor::enterX3DShapeNode(X3DShapeNode *N) 
{
  RendererStateVariables *stateVariables = Singleton<RendererStateVariables>::getInstance();
  if (stateVariables->getSelectionMode())
  {
    stateVariables->storeShape(N);
    glLoadName(stateVariables->getCurrentId());
//    cout << "loading name " << stateVariables->getCurrentId() << endl;
  }  
}

}
}
